package gfx;

import org.lwjgl.opengl.GL11;
import org.newdawn.slick.opengl.Texture;

/**
 * A simple implementation of fonts in OpenGL. All this actually
 * does it draw a bunch of quad's when the characters for the string
 * should be. It then textures those quads with a specially created
 * texture containing all the characters from a font. The texture
 * coordinates are chosen so that the appropriate bit of the texture
 * is shown on each quad for the characters.
 * 
 * Theres a really good tool for building textures containing fonts
 * called Bitmap Font Builder (http://www.lmnopc.com/bitmapfontbuilder/)
 * 
 * The tool actually produces textures containing two textures, one at the 
 * bottom and one at the top, which is what this class is based on
 * 
 * @author Kevin Glass
 */
public class BitmapFont {

	
	private static String chars = "" + //
			"ABCDEFGHIJKLMNOPQRSTUVWXYZ      " + //
			"0123456789.,!?'\"-+=/\\%()<>:;     " + //
			"";

	public BitmapFont() {
		
	}
	
	

		
	
	

	public void drawString( String msg, int x, int y, int size) {
		
		
	
		// bind the font text so we can render quads with the characters
		// on
	//	GL11.glEnable(GL11.GL_TEXTURE_2D);
		//texture.bind();
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, 2);
		// turn blending on so characters are displayed above the
		// scene
		GL11.glEnable(GL11.GL_BLEND);
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D,GL11.GL_TEXTURE_MAG_FILTER,GL11.GL_NEAREST);
		// cycle through each character drawing a quad to the screen
		// mapped to the right part of the texture
		GL11.glBegin(GL11.GL_QUADS);
		
		
		msg.toUpperCase();
		
		float Hoffsett = 0.0f;
		float Voffsett = 0.0f;
		float squaresize = 0.03125f;
		
		for (int i=0;i<msg.length();i++) {
			
			int ix = chars.indexOf(msg.charAt(i));
			if (ix >= 0) {
				
						
			Hoffsett = squaresize*ix;
			
			Voffsett = ((int)(ix/32))*squaresize;
			
			
			//start drawing
			GL11.glBegin(GL11.GL_QUADS);
			GL11.glTexCoord2f(0+Hoffsett,squaresize + Voffsett);
			GL11.glVertex2f(x+(i*size),y+size);
			
			
			GL11.glTexCoord2f(squaresize+Hoffsett,squaresize+Voffsett);
			GL11.glVertex2f(x+size+(i*size),y+size);
			
			
			GL11.glTexCoord2f(squaresize+Hoffsett,0+Voffsett);
			GL11.glVertex2f(x+size+(i*size),y);
			
			GL11.glTexCoord2f(0+Hoffsett,0+Voffsett);
			GL11.glVertex2f(x+(i*size),y);
			
	
			}
		}
		GL11.glEnd();
		
		// reset the blending
		GL11.glDisable(GL11.GL_BLEND);
	}
}

